#region usings
using System;
using System.ComponentModel.Composition;
 
using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;
 
using VVVV.Core.Logging;
#endregion usings
 
namespace VVVV.Nodes
{
    #region PluginInfo
    [PluginInfo(Name = "MultipleLinesIntersect", Category = "Value", Help = "2 lines intersect", Tags = "")]
    //Source:
    //http://paulbourke.net/geometry/lineline2d/
    //Credits:
    //Lasal 2011
 
    #endregion PluginInfo
    public class Value2LinesIntersectNode : IPluginEvaluate
    {
        #region fields & pins
 
                // Could it be possible to use a spread of points? 
 
                [Input("Line Start")]
        ISpread<Vector2D> P1;
 
        [Input("Line End")]
        ISpread<Vector2D> P2;
 
 
        [Output("Intersection")]
        ISpread<Vector2D> FIntersection;
 
        [Output("Number of Solutions")]
        ISpread<int> FSolutionCount;
 
        [Import()]
        ILogger FLogger;
        #endregion fields & pins
 
        //called when data for any output pin is requested
        public void Evaluate(int SpreadMax)
        {
            FIntersection.SliceCount = SpreadMax;
            FSolutionCount.SliceCount = SpreadMax;
 
            for(int i = 0; i < FIntersection.SliceCount; i++)
            {
               FIntersection.SliceCount = 0;
            }
 
            for (int i = 0; i < SpreadMax; i++)
                {
                for (int j = i + 1; j < SpreadMax; j++)
                {    
                    Vector2D s1;
                    Vector2D s2;
 
                //How to use 10 variables? P1[j], P2[j], P3[j], P4[j]
 
                FSolutionCount[i] = LinesIntersection( P1[i], P2[i], P1[j], P2[j], out s1, out s2);
 
                FIntersection.Add(s1);
                FIntersection.Add(s2);
                }
 
            }
 
 
        }
        private int LinesIntersection ( Vector2D P1 , Vector2D P2, Vector2D P3 , Vector2D P4, out Vector2D so1, out Vector2D so2)
        {
            so1 = new Vector2D();
            so2 = new Vector2D();
 
        // Denominator for ua and ub are the same, so store this calculation
         double d = 
            (P4.y - P3.y) * (P2.x - P1.x)
            -
            (P4.x - P3.x) * (P2.y - P1.y);
 
         //n_a and n_b are calculated as seperate values for readability
         double n_a =
            (P4.x - P3.x) * (P1.y - P3.y)
            -
            (P4.y - P3.y) * (P1.x - P3.x);
 
         double n_b =
            (P2.x - P1.x) * (P1.y - P2.y)
            -
            (P2.y - P1.y) * (P1.x - P2.x);
 
         // Calculate the intermediate fractional point that the lines potentially intersect.
         double ua = n_a / d;
         double ub = n_b / d;
 
         double   SX = P1.x + (ua * (P2.x - P1.x));
         double   SY = P1.y + (ua * (P2.y - P1.y));
 
        if (d == 0)   // Parallel lines.
        {
            return 0;
            }
 
        if (SX < P1.x & SX < P2.x | 
            SY < P1.y & SY < P2.y | 
            SX > P1.x & SX > P2.x | 
            SY > P1.y & SY > P2.y )  
            // No finite lines intersecting, delete this for infinite lines Intersection.
            {
            return 0;
            }
 
 
          // than 0 the lines would need to be longer to intersect.
         if (ua >= 0d && ua <= 1d && ub >= 0d && ub <= 1d)
         {
            so1.x = SX;
            so1.y = SY;
            return 1;
            }
 
 
            {
            return 0;    
			} 
	}
    
    }
	
}